@@ -237,15 +237,18 @@ impl LSTM {
         let chunk = if dir == 0 { 1 } else { -1 };
 
         // X: onnx interface: [seq_length, batch_size, input_size]
+        // move batch first
+        target_wire!(x_batch_first = AxisOp::Move(1, 0), inputs[0]);
+        // X: onnx interface: [batch_size, seq_length, input_size]
         // scan outer interface: idem
-        // scann inner interface: [chunk=1, batch_size, input_size]
+        // scann inner interface: [batch_size, chunk=1, input_size]
         // onnx inner interface: [batch_size, input_size]
-        outer_inputs.push(inputs[0]);
-        input_mapping.push(scan::InputMapping::Scan { slot: 0, axis: 0, chunk });
-        let mut x_source_fact = x_fact.without_value();
-        x_source_fact.shape.set(0, 1.to_dim());
+        outer_inputs.push(x_batch_first);
+        input_mapping.push(scan::InputMapping::Scan { slot: 0, axis: 1, chunk });
+        let mut x_source_fact = target.outlet_fact(x_batch_first)?.without_value();
+        x_source_fact.shape.set(1, 1.to_dim());
         let x_source = body.add_source("x_source", x_source_fact)?.into();
-        wire!(Xt = AxisOp::Rm(0), x_source);
+        wire!(Xt = AxisOp::Rm(1), x_source);
 
         // W: onnx interface: [num_directions, 4*hidden_size, input_size]
         // scan interfaces: [4*hidden_size, input_size]
@@ -279,21 +282,22 @@ impl LSTM {
         }
 
         // initial h, optional: onnx: [num_directions, batch_size, hidden_size]
-        // scan outer: [chunk=1, batch_size, hidden_size]
-        // scan inner: [chunk=1, batch_size, hidden_size]
+        // scan outer: [batch_size, chunk=1, hidden_size]
+        // scan inner: [batch_size, chunk=1, hidden_size]
         // onnx inner: [batch_size, hidden_size]
         let initializer = if let Some(initial_h_input) = self.optional_initial_h_input {
             target_wire!(h_dir = array::Slice::new(0, dir, dir + 1), inputs[initial_h_input]);
             target_wire!(h = AxisOp::Rm(0), h_dir);
-            target_wire!(h_chunk = AxisOp::Add(0), h);
+            target_wire!(h_chunk_ = AxisOp::Add(0), h);
+            target_wire!(h_chunk = AxisOp::Move(1, 0), h_chunk_);
             outer_inputs.push(h_chunk);
             scan::StateInitializer::FromInput(initial_h_input)
         } else {
             scan::StateInitializer::Value(
                 tensor0(0.0f32)
                     .broadcast_scalar_to_shape(&[
-                        1,
                         b_size.to_usize().unwrap(),
+                        1,
                         h_size.to_usize().unwrap(),
                     ])?
                     .into_arc_tensor(),
@@ -305,7 +309,7 @@ impl LSTM {
                 "h_source",
                 TypedFact::dt_shape(
                     x_fact.datum_type,
-                    &[1.to_dim(), b_size.clone(), h_size.clone()],
+                    &[b_size.clone(), 1.to_dim(), h_size.clone()],
                 ),
             )?
             .into();
@@ -313,15 +317,16 @@ impl LSTM {
         let initializer = if let Some(initial_c_input) = self.optional_initial_c_input {
             target_wire!(c_dir = array::Slice::new(0, dir, dir + 1), inputs[initial_c_input]);
             target_wire!(c = AxisOp::Rm(0), c_dir);
-            target_wire!(c_chunk = AxisOp::Add(0), c);
+            target_wire!(c_chunk_ = AxisOp::Add(0), c);
+            target_wire!(c_chunk = AxisOp::Move(1, 0), c_chunk_);
             outer_inputs.push(c_chunk);
             scan::StateInitializer::FromInput(initial_c_input)
         } else {
             scan::StateInitializer::Value(
                 tensor0(0.0f32)
                     .broadcast_scalar_to_shape(&[
-                        1,
                         b_size.to_usize().unwrap(),
+                        1,
                         h_size.to_usize().unwrap(),
                     ])?
                     .into_arc_tensor(),
@@ -333,7 +338,7 @@ impl LSTM {
                 "c_source",
                 TypedFact::dt_shape(
                     x_fact.datum_type,
-                    &[1.to_dim(), b_size.clone(), h_size.clone()],
+                    &[b_size.clone(), 1.to_dim(), h_size.clone()],
                 ),
             )?
             .into();
@@ -349,8 +354,11 @@ impl LSTM {
             None
         };
 
-        wire!(Ht_1 = AxisOp::Rm(0), h_source);
-        wire!(Ct_1 = AxisOp::Rm(0), c_source);
+        // drop sequence axis (chunk == 1)
+        wire!(Ht_1 = AxisOp::Rm(1), h_source);
+        // onnx inner: [batch_size, hidden_size]
+        wire!(Ct_1 = AxisOp::Rm(1), c_source);
+        // onnx inner: [batch_size, hidden_size]
 
         wire!(Wi = array::Slice::new(0, 0.to_dim() * h_size, 1.to_dim() * h_size), W);
         wire!(Wo = array::Slice::new(0, 1.to_dim() * h_size, 2.to_dim() * h_size), W);
@@ -460,13 +468,15 @@ impl LSTM {
         wire!(h_Ct = self.h.clone(), Ct);
         wire!(Ht = math::mul::bin_typed(), ot, h_Ct);
 
-        wire!(Ht_fixed = AxisOp::Add(0), Ht);
-        wire!(Ct_fixed = AxisOp::Add(0), Ct);
+        // onnx inner interface: [batch_size, input_size]
+        // add sequence axis (chunk == 1)
+        wire!(Ht_fixed = AxisOp::Add(1), Ht);
+        wire!(Ct_fixed = AxisOp::Add(1), Ct);
         body.set_output_outlets(&[Ht_fixed, Ct_fixed])?;
 
         let h_mapping = scan::OutputMapping {
             state: true,
-            axis: 0,
+            axis: 1,
             chunk,
             full_dim_hint: None,
             last_value_slot: self.optional_y_h_output,
@@ -474,7 +484,7 @@ impl LSTM {
         };
         let c_mapping = scan::OutputMapping {
             state: true,
-            axis: 0,
+            axis: 1,
             chunk,
             full_dim_hint: None,
             last_value_slot: self.optional_y_c_output,
@@ -495,14 +505,17 @@ impl LSTM {
 
         let mut result = tvec!();
         if let Some(slot) = self.optional_y_output {
-            target_wire!(y = AxisOp::Add(1), scan_outputs[slot]);
+            target_wire!(y_batch_middle = AxisOp::Move(1, 0), scan_outputs[slot]);
+            target_wire!(y = AxisOp::Add(1), y_batch_middle);
             result.push(y);
         }
         if let Some(slot) = self.optional_y_h_output {
-            result.push(scan_outputs[slot]);
+            target_wire!(y_h_batch_middle = AxisOp::Move(1, 0), scan_outputs[slot]);
+            result.push(y_h_batch_middle);
         }
         if let Some(slot) = self.optional_y_c_output {
-            result.push(scan_outputs[slot]);
+            target_wire!(y_c_batch_middle = AxisOp::Move(1, 0), scan_outputs[slot]);
+            result.push(y_c_batch_middle);
         }
 
         Ok(result)
